%  An example of computing the Ruppert, Nettleton, Hwang unweighted and weighted semiparametric estimators and the convex decreasing
%  estimator.

%  Here p is a vector of p-values

nobs = 4000 ; % number of p-values
nknots = 12 ; % number of knots
p = rand(nobs,1) ; % In the example, the p-values are random uniforms
lambdas = logspace(-2,6,6) ; % vector of possible penalty parameters
penalty_type = 2 % type of penalty (keep at 2)
test_type = 3 ;  % 1=z-test, 2=Wilcoxon rank-sum, 3=t-test
if test_type == 3 ;
    ittest = 1 ; % indicates a t-test
end ;
DF_t = 4 ; ; % the program will run fast only if df = 4, 8, or 20 because the noncentral-t quantiles are stored on disk for these
         % values.  For other values of t, these quantiles must be computed (alternatively, one 
         % store the quantiles for additional degrees of
         % freedom.)
         
df = DF_t ;
twosided = 1 ; % 1 = two-sided test, 0 = one-sided test

GCVfact = 1 ;


nbin_semi = 2000 ;  % initialize number of bins for semiparametric estimators
nbin_cd = 2000 ;  % initialize number of bins for convex, decreasing estimator

edges = linspace(eps,1-eps,nbin_semi+1)' ;  % compute bin counts
counts = histc(p,edges) ;
counts = counts(1:nbin_semi) ;


% splinefit02 computes the Ruppert, Nettleton, Hwang estimator



fit_unwt = splinefit02(counts,nobs,edges,struct('nknots',nknots,'lambdas',lambdas,'twosided',twosided,...
    'wt',ones(nbin_semi,1),'GCVfact',GCVfact,'penalty_type',penalty_type,'nbin',nbin_semi,'ittest',ittest,'df',df)) ;
% fit_unwt.betahatmin1, fit_unwt.fhatmin, fit_unwt.fhat12min are the
% SemiPar, fhatmin, and comprise estimator (all unweighted)


wt1 = 1./sqrt(fit_unwt.fhat) ;

fit_wt = splinefit02(counts,nobs,edges,struct('nknots',nknots,'lambdas',lambdas,'twosided',twosided,...
    'wt',wt1,'GCVfact',GCVfact,'penalty_type',penalty_type,'nbin',nbin_semi,'ittest',ittest,'df',df)) ;


% convexdecrdensity01 computes the convex, decreasing estimator

paramcd = struct('nbin',nbin_cd) ;
fitcd0 = convexdecrdensity01(p,paramcd) ;

figure(1)
plot(fit_wt.bincenters,fit_wt.fhat,fit_wt.bincenters,fitcd0.fhat,'--')
legend('semiparametric','convex, decreasing')