function fit = convexdecrdensity01(p,parameters) ;

if nargin == 1 ;
    parameters = [] ;
end ;

if isfield(parameters,'nbin') == 0 ;
    nbin = 2000 ;
else ;
    nbin = parameters.nbin ;
end ;

if isfield(parameters,'nknots') == 0 ;
    nknots = 100 ;
else ;
    nknots = parameters.nknots ;
end ;

if isfield(parameters,'wt') == 0 ;
    wt = ones(nbin,1) ;
else ;
    wt = parameters.wt ;
end ;

nobs = length(p) ;
edges = linspace(0,1,nbin+1)' ;
binwidths = edges(2:nbin+1) - edges(1:nbin) ;
binwid = binwidths(1) ;
bincenters = (edges(2:nbin+1) + edges(1:nbin))/2 ;
counts = histc(p,edges) ;
counts = counts(1:nbin) ;
knots = (1:nknots)/nknots ;
one = 1 + 0*bincenters ;
XX = one ;
for i=1:nknots ;
    XX = [XX  (2/knots(i)^2)*(knots(i)*one - bincenters).*(knots(i)*one > bincenters)] ;
end ;
XXwt = XX.*(wt*ones(1,size(XX,2))) ;
y = counts./(nobs*binwidths) ;
y_unwt = y ;
y = y.*wt ;
XXXXwt = XXwt'*XXwt ;
yXXwt = y'*XXwt ;
LB = zeros(nknots+1) ;
UB = [] ;
Aeq = ones(1,nknots+1) ;
Beq = 1 ;
f = -yXXwt ;
H = XXXXwt ;
theta = quadprog(H,f,[],[],Aeq,Beq,LB,UB) ;
fhat = XX*theta ;

fit = struct('theta',theta,'fhat',fhat,'pzero',fhat(nbin),'bincenters',bincenters,'y_unwt',y_unwt,'counts',counts,...
    'binwidths',binwidths) ;

%     X=QUADPROG(H,f,A,b,Aeq,beq) solves the problem above while additionally
%     satisfying the equality constraints Aeq*x = beq.
    
%     X=QUADPROG(H,f,A,b,Aeq,beq,LB,UB) defines a set of lower and upper
%     bounds on the design variables, X, so that the solution is in the 
%     range LB <= X <= UB. Use empty matrices for LB and UB
%     if no bounds exist. Set LB(i) = -Inf if X(i) is unbounded below; 
%     set UB(i) = Inf if X(i) is unbounded above.




