function fit = splinefit02(counts,nobs,edges,parameters) ;

if nargin == 1 ;
    parameters = [] ;
end ;

if isfield(parameters,'ttest') == 0 ;
    ttest = 0 ;
    df = 1000 ;
else ;
    ttest = parameters.ttest ;
    df = parameters.df ;
end ;

if isfield(parameters,'twosided') == 0 ;
    twosided = 1 ;
else ;
    twosided = parameters.twosided ;
end ;


if isfield(parameters,'lambdas') == 0 ;
    lambdas =  logspace(-4,5,8) ;
else ;
    lambdas = parameters.lambdas ;
end ;

if isfield(parameters,'nbin') == 0 ;
    nbin = 2000 ;
else ;
    nbin = parameters.nbin ;
end ;

if isfield(parameters,'nknots') == 0 ;
    nknots = 8 ;
else ;
    nknots = parameters.nknots ;
end ;

if isfield(parameters,'nmu') == 0 ;
    nmu = 500 ;
else ;
    nmu = parameters.nmu ;
end ;

if isfield(parameters,'GCVfact') == 0 ;
    GCVfact = 1 ;
else ;
    GCVfact = parameters.GCVfact ;
end ;

if isfield(parameters,'nskip') == 0 ;
    nskip = 0 ;
else ;
    nskip = parameters.nskip ;
end ;

if isfield(parameters,'wt') == 0 ;
    wt = ones(nbin,1) ;
else ;
    wt = parameters.wt ;
end ;

if isfield(parameters,'penalty_type') == 0 ;
    penalty_type = 1 ;
else ;
    penalty_type = parameters.penalty_type ;
end ;

if isfield(parameters,'mumax') == 0 ;
    mumax = 6 ;
else ;
    mumax = parameters.mumax ;
end ;

binnum = (1:nbin)';
wt = wt .* (binnum > nskip) ;

binwidths = edges(2:nbin+1) - edges(1:nbin) ;
binwid = binwidths(1) ;
bincenters = (edges(2:nbin+1) + edges(1:nbin))/2 ;

XX = 1 + 0*binwidths ;
mu = linspace(eps,mumax,nmu)' ;
knots = linspace(0,mumax,nknots)' ;
[b,newknots]= Bsplines01(mu,knots,1);
b2 = 2*b(:,2:size(b,2)-3) ;
cdfp = cdfp01(edges,mu,twosided,ttest,df) ;
dif_cdfp = cdfp(2:nbin+1,:) - cdfp(1:nbin,:) ;
KK = size(b2,2) ;
b2 = [ones(nmu,1) b2] ;
for j=1:KK ;
    newcol = dif_cdfp*b2(:,j+1) ;  % changed 'j' to 'j+1' on 9/28/05
    newcol = nbin*newcol/sum(newcol) ;
    b2(:,j+1) = nbin* b2(:,j+1) /sum(newcol) ;
    XX = [XX newcol ];
end ;
XXwt = XX.*(wt*ones(1,size(XX,2))) ;
II = eye(KK+1) ;
II(2,2) = 2 ;

if penalty_type == 1 ;
    D = zeros(nknots,nknots) ;
    for j=2:nknots-1 ;
        D(j,j) = 1 ;
        D(j,j+1) = -1 ;
    end ;
elseif penalty_type == 2 ;
    D = zeros(nknots-3,nknots) ;
    for j=1:nknots-3 ;
        D(j,j+1) = 1 ;
        D(j,j+2) = -2 ;
        D(j,j+3) = 1 ;
    end ;
end ;
DD = II*D'*D*II ;
y = counts./(nobs*binwidths) ;
ywt = y.*wt ;
XXXXwt = XXwt'*XXwt ;
yXXwt = ywt'*XXwt ;
LB = zeros(nknots) ;
UB = [] ;
Aeq = ones(1,nknots) ;
Beq = 1 ;
yywt = ywt'*ywt ;
for ilam = 1:length(lambdas) ;
    lambda = lambdas(ilam) ;
    BB = ( XXXXwt + lambda*D'*D) ;
    f = -yXXwt ;
    H = XXXXwt + lambda*DD ;
    thetanew = quadprog(H,f,[],[],Aeq,Beq,LB,UB) ;
    betahat(ilam,:) = thetanew ;
    df(ilam) = trace( inv(BB)*XXXXwt )  ;
    p0hat(ilam) = betahat(ilam,1) ;
%     res = y-XXwt*betahat(ilam,:)' ;
    gcv(ilam) = ( yywt -2*yXXwt*thetanew + thetanew'*XXXXwt*thetanew  ) / (nbin-nskip - GCVfact*df(ilam))^2 ;
end ;
imin = max(find(gcv == min(gcv))) ;
betahatmin = betahat(imin,:) ;
fhat = XX*betahatmin' ;

comprise_n = 2 ;

fhat12 = XX(:,1:3)*betahatmin(1:3)' ;
fhat12min = min(fhat12) ;
fhatmin = min(fhat) ;
betahatmin1 = betahatmin(1) ;
ghat = b2(:,2:size(b2,2)) * betahatmin(2:length(betahatmin))' ;
ghat = ghat / (1-betahatmin(1)) ;
fhatp0 = betahatmin1 + 0 * fhat ;

fit = struct('betahat',betahat,'betahatmin',betahatmin,'fhat',fhat,'fhatmin',fhatmin,...
    'df',df,'p0hat',p0hat','gcv',gcv,'imin',imin,'XXXXwt',XXXXwt,'D',D,...
    'lambdas',lambdas,'betahatmin1',betahatmin1,...
    'b',b,'knots',knots,'mu',mu,...
    'bincenters',bincenters,'binwidths',binwidths,'counts',counts,...
    'ghat',ghat,'b2',b2,'GCVfact',GCVfact,'wt',wt,'DD',DD,'II',II,...
    'XX',XX,'fhat12min',fhat12min,'fhat12',fhat12,'fhatp0',fhatp0,'cdfp',cdfp) ;







